@@ -1,7 +1,7 @@
 /* mpfr_setmax -- maximum representable floating-point number (raw version)
 
-Copyright 2002-2004, 2006-2015 Free Software Foundation, Inc.
-Contributed by the AriC and Caramel projects, INRIA.
+Copyright 2002-2004, 2006-2020 Free Software Foundation, Inc.
+Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -17,7 +17,7 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #include "mpfr-impl.h"
@@ -35,7 +35,7 @@ mpfr_setmax (mpfr_ptr x, mpfr_exp_t e)
   xn = MPFR_LIMB_SIZE (x);
   sh = (mpfr_prec_t) xn * GMP_NUMB_BITS - MPFR_PREC(x);
   xp = MPFR_MANT(x);
-  xp[0] = MP_LIMB_T_MAX << sh;
+  xp[0] = MPFR_LIMB_MAX << sh;
   for (i = 1; i < xn; i++)
-    xp[i] = MP_LIMB_T_MAX;
+    xp[i] = MPFR_LIMB_MAX;
 }
