@@ -1,7 +1,7 @@
 /* Test compatibility mpf-mpfr.
 
-Copyright 2003, 2005-2015 Free Software Foundation, Inc.
-Contributed by the AriC and Caramel projects, INRIA.
+Copyright 2003, 2005-2020 Free Software Foundation, Inc.
+Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -17,9 +17,23 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
+#include "mpfr-test.h"
+
+#ifndef MPFR_USE_MINI_GMP
+
 #define MPF
 #define mpf_free_str mpfr_free_str
 #include "mpf_compat.h"
+
+#else
+
+int
+main (void)
+{
+  return 77;
+}
+
+#endif
