@@ -1,6 +1,6 @@
 /* Test file for mpfr_cmp2.
 
-Copyright 1999-2003, 2006-2016 Free Software Foundation, Inc.
+Copyright 1999-2003, 2006-2020 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -17,12 +17,9 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdio.h>
-#include <stdlib.h>
-
 #include "mpfr-test.h"
 
 /* set bit n of x to b, where bit 0 is the most significant one */
@@ -37,9 +34,9 @@ set_bit (mpfr_t x, unsigned int n, int b)
   n %= mp_bits_per_limb;
   n = mp_bits_per_limb - 1 - n;
   if (b)
-    MPFR_MANT(x)[xn - l] |= (mp_limb_t) 1 << n;
+    MPFR_MANT(x)[xn - l] |= MPFR_LIMB_ONE << n;
   else
-    MPFR_MANT(x)[xn - l] &= ~((mp_limb_t) 1 << n);
+    MPFR_MANT(x)[xn - l] &= ~(MPFR_LIMB_ONE << n);
 }
 
 /* check that for x = 1.u 1 v 0^k low(x)
@@ -60,7 +57,7 @@ worst_cases (void)
   for (i = 1; i < MPFR_PREC(x); i++)
     {
       mpfr_set_ui (x, 1, MPFR_RNDN);
-      mpfr_div_2exp (y, y, 1, MPFR_RNDN); /* y = 1/2^i */
+      mpfr_div_2ui (y, y, 1, MPFR_RNDN); /* y = 1/2^i */
 
       l = 0;
       if (mpfr_cmp2 (x, y, &l) <= 0 || l != 1)
@@ -199,11 +196,9 @@ special (void)
     {
       printf ("Error in mpfr_cmp2:\n");
       printf ("x=");
-      mpfr_print_binary (x);
-      puts ("");
+      mpfr_dump (x);
       printf ("y=");
-      mpfr_print_binary (y);
-      puts ("");
+      mpfr_dump (y);
       printf ("got %lu, expected 1\n", (unsigned long) j);
       exit (1);
     }
@@ -215,8 +210,8 @@ special (void)
   if (mpfr_cmp2(x, y, &j) <= 0 || j != 32)
     {
       printf ("Error in mpfr_cmp2:\n");
-      printf ("x="); mpfr_print_binary(x); puts ("");
-      printf ("y="); mpfr_print_binary(y); puts ("");
+      printf ("x="); mpfr_dump (x);
+      printf ("y="); mpfr_dump (y);
       printf ("got %lu, expected 32\n", (unsigned long) j);
       exit (1);
     }
@@ -228,8 +223,8 @@ special (void)
   if (mpfr_cmp2(x, y, &j) <= 0 || j != 164)
     {
       printf ("Error in mpfr_cmp2:\n");
-      printf ("x="); mpfr_print_binary(x); puts ("");
-      printf ("y="); mpfr_print_binary(y); puts ("");
+      printf ("x="); mpfr_dump (x);
+      printf ("y="); mpfr_dump (y);
       printf ("got %lu, expected 164\n", (unsigned long) j);
       exit (1);
     }
@@ -242,8 +237,8 @@ special (void)
   if (mpfr_cmp2(x, y, &j) <= 0 || j != 127)
     {
       printf ("Error in mpfr_cmp2:\n");
-      printf ("x="); mpfr_print_binary(x); puts ("");
-      printf ("y="); mpfr_print_binary(y); puts ("");
+      printf ("x="); mpfr_dump (x);
+      printf ("y="); mpfr_dump (y);
       printf ("got %lu, expected 127\n", (unsigned long) j);
       exit (1);
     }
@@ -256,8 +251,8 @@ special (void)
   if (mpfr_cmp2(x, y, &j) <= 0 || j != 63)
     {
       printf ("Error in mpfr_cmp2:\n");
-      printf ("x="); mpfr_print_binary(x); puts ("");
-      printf ("y="); mpfr_print_binary(y); puts ("");
+      printf ("x="); mpfr_dump (x);
+      printf ("y="); mpfr_dump (y);
       printf ("got %lu, expected 63\n", (unsigned long) j);
       exit (1);
     }
@@ -270,8 +265,8 @@ special (void)
   if (mpfr_cmp2(x, y, &j) <= 0 || j != 63)
     {
       printf ("Error in mpfr_cmp2:\n");
-      printf ("x="); mpfr_print_binary(x); puts ("");
-      printf ("y="); mpfr_print_binary(y); puts ("");
+      printf ("x="); mpfr_dump (x);
+      printf ("y="); mpfr_dump (y);
       printf ("got %lu, expected 63\n", (unsigned long) j);
       exit (1);
     }
@@ -302,6 +297,51 @@ special (void)
   mpfr_clear (y);
 }
 
+/* Compare (m,kx) and (m,ky), where (m,k) means m fixed limbs followed by
+   k zero limbs. */
+static void
+test_equal (void)
+{
+  mpfr_t w, x, y;
+  int m, kx, ky, inex;
+  mpfr_prec_t j;
+
+  for (m = 1; m <= 4; m++)
+    {
+      mpfr_init2 (w, m * GMP_NUMB_BITS);
+      for (kx = 0; kx <= 4; kx++)
+        for (ky = 0; ky <= 4; ky++)
+          {
+            do mpfr_urandomb (w, RANDS); while (mpfr_zero_p (w));
+            mpfr_init2 (x, (m + kx) * GMP_NUMB_BITS
+                        - (kx == 0 ? 0 : randlimb () % GMP_NUMB_BITS));
+            mpfr_init2 (y, (m + ky) * GMP_NUMB_BITS
+                        - (ky == 0 ? 0 : randlimb () % GMP_NUMB_BITS));
+            inex = mpfr_set (x, w, MPFR_RNDN);
+            MPFR_ASSERTN (inex == 0);
+            inex = mpfr_set (y, w, MPFR_RNDN);
+            MPFR_ASSERTN (inex == 0);
+            MPFR_ASSERTN (mpfr_equal_p (x, y));
+            if (randlimb () & 1)
+              mpfr_neg (x, x, MPFR_RNDN);
+            if (randlimb () & 1)
+              mpfr_neg (y, y, MPFR_RNDN);
+            if (mpfr_cmp2 (x, y, &j) != 0)
+              {
+                printf ("Error in test_equal for m = %d, kx = %d, ky = %d\n",
+                        m, kx, ky);
+                printf ("  x = ");
+                mpfr_dump (x);
+                printf ("  y = ");
+                mpfr_dump (y);
+                exit (1);
+              }
+            mpfr_clears (x, y, (mpfr_ptr) 0);
+          }
+      mpfr_clear (w);
+    }
+}
+
 int
 main (void)
 {
@@ -342,6 +382,8 @@ main (void)
         tcmp2 (x, y, -1);
     }
 
+  test_equal ();
+
   tests_end_mpfr ();
 
   return 0;
