@@ -1,7 +1,7 @@
 /* Test file for mpfr_const_euler.
 
-Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Free Software Foundation, Inc.
-Contributed by the AriC and Caramel projects, INRIA.
+Copyright 2001-2020 Free Software Foundation, Inc.
+Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -17,12 +17,9 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdio.h>
-#include <stdlib.h>
-
 #include "mpfr-test.h"
 
 /* Wrapper for tgeneric */
@@ -36,6 +33,22 @@ my_const_euler (mpfr_ptr x, mpfr_srcptr y, mpfr_rnd_t r)
 #define TEST_FUNCTION my_const_euler
 #include "tgeneric.c"
 
+static void
+exercise_Ziv (void)
+{
+  mpfr_t x, y;
+  int inex;
+
+  mpfr_init2 (x, 177);
+  mpfr_init2 (y, 177);
+  inex = mpfr_const_euler (x, MPFR_RNDN);
+  mpfr_set_str_binary (y, "0.10010011110001000110011111100011011111011011000011000111101001001101000110111110001111111000000100000001010100101100101101010110101000011100111011001100001110101111011001011101");
+  MPFR_ASSERTN(mpfr_equal_p (x, y));
+  MPFR_ASSERTN(inex > 0);
+  mpfr_clear (x);
+  mpfr_clear (y);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -57,6 +70,8 @@ main (int argc, char *argv[])
       return 0;
     }
 
+  exercise_Ziv ();
+
   mpfr_init (y);
   mpfr_init (z);
   mpfr_init (t);
@@ -77,11 +92,14 @@ main (int argc, char *argv[])
       mpfr_set_prec (t, prec);
       yprec = prec + 10;
 
-      for (rnd = 0; rnd < MPFR_RND_MAX; rnd++)
+      RND_LOOP_NO_RNDF (rnd)
         {
           mpfr_set_prec (y, yprec);
           mpfr_const_euler (y, (mpfr_rnd_t) rnd);
           err = (rnd == MPFR_RNDN) ? yprec + 1 : yprec;
+          /* Note: for rnd = RNDF, rnd1 = RNDF is equivalent to rnd1 = RNDN
+             in mpfr_can_round, thus rnd2 = RNDF reduces to rnd2 = RNDN in that
+             case, we are duplicating the test for rnd = RNDN. */
           if (mpfr_can_round (y, err, (mpfr_rnd_t) rnd, (mpfr_rnd_t) rnd, prec))
             {
               mpfr_set (t, y, (mpfr_rnd_t) rnd);
@@ -97,8 +115,7 @@ main (int argc, char *argv[])
                   mpfr_out_str (stdout, 2, prec, t, MPFR_RNDN);
                   puts ("");
                   printf ("   approximation was ");
-                  mpfr_print_binary (y);
-                  puts ("");
+                  mpfr_dump (y);
                   exit (1);
                 }
             }
@@ -109,7 +126,7 @@ main (int argc, char *argv[])
   mpfr_clear (z);
   mpfr_clear (t);
 
-  test_generic (2, 200, 1);
+  test_generic (MPFR_PREC_MIN, 200, 1);
 
   tests_end_mpfr ();
   return 0;
