@@ -1,6 +1,6 @@
 /* Miscellaneous support for test programs.
 
-Copyright 2001-2017 Free Software Foundation, Inc.
+Copyright 2001-2020 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -17,7 +17,7 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #ifdef HAVE_CONFIG_H
@@ -25,7 +25,6 @@ http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 #endif
 
 #include <float.h>
-#include <errno.h>
 
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
@@ -154,7 +153,6 @@ test_version (void)
        (sprintf (buffer, "%d.%d", __GNU_MP_VERSION, __GNU_MP_VERSION_MINOR),
         strcmp (buffer, gmp_version) != 0)))
     err = 1;
-#endif
 
   /* In some cases, it may be acceptable to have different versions for
      the header and the library, in particular when shared libraries are
@@ -174,7 +172,8 @@ test_version (void)
       printf ("  * A bad configuration in your include/library search paths.\n"
               "  * An inconsistency in the include/library search paths of\n"
               "    your development environment; an example:\n"
-              "      https://gcc.gnu.org/ml/gcc-help/2010-11/msg00359.html\n"
+              "      "
+              "https://gcc.gnu.org/legacy-ml/gcc-help/2010-11/msg00359.html\n"
               "  * GMP has been upgraded after the first \"make check\".\n"
               "    In such a case, try again after a \"make clean\".\n"
               "  * A new or non-standard version naming is used in GMP.\n"
@@ -184,6 +183,7 @@ test_version (void)
               " with MPFR.\nIf some other tests fail, please solve that"
               " problem first.\n");
     }
+#endif
 
   /* VL: I get the following error on an OpenSUSE machine, and changing
      the value of shlibpath_overrides_runpath in the libtool file from
@@ -191,7 +191,6 @@ test_version (void)
   version = mpfr_get_version ();
   if (strcmp (MPFR_VERSION_STRING, version) == 0)
     {
-      char buffer[16];
       int i;
 
       sprintf (buffer, "%d.%d.%d", MPFR_VERSION_MAJOR, MPFR_VERSION_MINOR,
@@ -217,13 +216,16 @@ test_version (void)
       "  * On some platforms such as Solaris, $LD_LIBRARY_PATH overrides\n"
       "    the rpath, and if the MPFR library is already installed in a\n"
       "    $LD_LIBRARY_PATH directory, you typically get this error.  Do\n"
-      "    not use $LD_LIBRARY_PATH on such platforms; it may also break\n"
-      "    other things.\n"
+      "    not use $LD_LIBRARY_PATH permanently on such platforms; it may\n"
+      "    also break other things.\n"
       "  * You may have an ld option that specifies a library search path\n"
       "    where MPFR can be found, taking the precedence over the path\n"
       "    added by libtool.  Check your environment variables, such as\n"
-      "    LD_OPTIONS under Solaris.\n"
-      "  * Then look at http://www.mpfr.org/mpfr-current/ for any update.\n"
+      "    LD_OPTIONS under Solaris.  Moreover, under Solaris, the run path\n"
+      "    generated by libtool 2.4.6 may be incorrect: the build directory\n"
+      "    may not appear first in the run path; set $LD_LIBRARY_PATH to\n"
+      "    /path/to/builddir/src/.libs for the tests as a workaround.\n"
+      "  * Then look at https://www.mpfr.org/mpfr-current/ for any update.\n"
       "  * Try again on a completely clean source (some errors might come\n"
       "    from a previous build or previous source changes).\n"
       "  * If the error still occurs, you can try to change the value of\n"
@@ -235,6 +237,9 @@ test_version (void)
   /* Note about $LD_LIBRARY_PATH under Solaris:
    *   https://en.wikipedia.org/wiki/Rpath#Solaris_ld.so
    * This cause has been confirmed by a user who got this error.
+   * And about the libtool 2.4.6 bug also concerning Solaris:
+   *   https://debbugs.gnu.org/cgi/bugreport.cgi?bug=30222
+   *   https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=888059
    */
   exit (1);
 }
@@ -250,9 +255,16 @@ void
 tests_start_mpfr (void)
 {
   /* Don't buffer, so output is not lost if a test causes a segv, etc.
-     Warning! No operations must have already been done on stdout
-     (this is a requirement of ISO C, and this is important on AIX). */
+     For stdout, this is important as it will typically be fully buffered
+     by default with "make check". For stderr, the C standard just says
+     that it is not fully buffered (it may be line buffered by default);
+     disabling buffering completely might be useful in some cases.
+     Warning! No operations must have already been done on stdout/stderr
+     (this is a requirement of ISO C, and this is important on AIX).
+     Thus tests_start_mpfr should be called at the beginning of main(),
+     possibly after some variable settings. */
   setbuf (stdout, NULL);
+  setbuf (stderr, NULL);
 
   test_version ();
 
@@ -398,7 +410,13 @@ tests_rand_start (void)
 #ifdef HAVE_GETTIMEOFDAY
           struct timeval  tv;
           gettimeofday (&tv, NULL);
-          seed = tv.tv_sec + tv.tv_usec;
+          /* Note: If time_t is a "floating type" (as allowed by ISO C99),
+             the cast below can yield undefined behavior. But this would
+             be uncommon (gettimeofday() is specified by POSIX only and
+             POSIX requires time_t to be an integer type) and this line
+             is not executed by default. So, this should be OK. Moreover,
+             gettimeofday() is marked obsolescent by POSIX.1-2008. */
+          seed = 1000000 * (unsigned long) tv.tv_sec + tv.tv_usec;
 #else
           time_t  tv;
           time (&tv);
@@ -433,12 +451,12 @@ mpfr_test_init (void)
   set_fpc_csr(exp.fc_word);
 #endif
 
-#ifdef HAVE_DENORMS
+#ifdef HAVE_SUBNORM_DBL
   {
     double d = DBL_MIN;
     if (2.0 * (d / 2.0) != d)
       {
-        printf ("Error: HAVE_DENORMS defined, but no subnormals.\n");
+        printf ("Error: HAVE_SUBNORM_DBL defined, but no subnormals.\n");
         exit (1);
       }
   }
@@ -575,6 +593,30 @@ ld_trace (const char *name, long double ld)
   printf ("] %.20Lg\n", ld);
 }
 
+void
+n_trace (const char *name, mp_limb_t *p, mp_size_t n)
+{
+  unsigned char *buf;
+  size_t bufsize;
+  mp_size_t i, m;
+
+  if (name != NULL && name[0] != '\0')
+    printf ("%s=", name);
+
+  /* similar to gmp_printf ("%NX\n",...), which is not available
+     with mini-gmp */
+  bufsize = 2 + ((mpfr_prec_t) n * GMP_NUMB_BITS - 1) / 4;
+  buf = (unsigned char *) tests_allocate (bufsize);
+  m = mpn_get_str (buf, 16, p, n);
+  i = 0;
+  while (i < m - 1 && buf[i] == 0)
+    i++;  /* skip leading zeros (keeping at least one digit) */
+  while (i < m)
+    putchar ("0123456789ABCDEF"[buf[i++]]);
+  putchar ('\n');
+  tests_free (buf, bufsize);
+}
+
 /* Open a file in the SRCDIR directory, i.e. the "tests" source directory,
    which is different from the current directory when objdir is different
    from srcdir. One should generally use this function instead of fopen
@@ -662,28 +704,35 @@ tests_default_random (mpfr_ptr x, int pos, mpfr_exp_t emin, mpfr_exp_t emax,
     mpfr_neg (x, x, MPFR_RNDN);
 }
 
-/* The test_one argument is seen a boolean. If it is true and rnd is
-   a rounding mode toward infinity, then the function is tested in
-   only one rounding mode (the one provided in rnd) and the variable
-   rndnext is not used (due to the break). If it is true and rnd is a
-   rounding mode toward or away from zero, then the function is tested
-   twice, first with the provided rounding mode and second with the
-   rounding mode toward the corresponding infinity (determined by the
-   sign of the result). If it is false, then the function is tested
-   in the 5 rounding modes, and rnd must initially be MPFR_RNDZ; thus
-   rndnext will be initialized in the first iteration.
-   If the test_one argument is 2, then this means that y is exact, and
-   the ternary value is checked.
+/* If sb = -1, then the function is tested in only one rounding mode
+   (the one provided in rnd) and the ternary value is not checked.
+   Otherwise, the function is tested in the 5 rounding modes, rnd must
+   initially be MPFR_RNDZ, y = RNDZ(f(x)), and sb is 0 if f(x) is exact,
+   1 if f(x) is inexact (in which case, y must be a regular number,
+   i.e. not the result of an overflow or an underflow); the successive
+   rounding modes are:
+     * MPFR_RNDZ, MPFR_RNDD, MPFR_RNDA, MPFR_RNDU, MPFR_RNDN for positive y;
+     * MPFR_RNDZ, MPFR_RNDU, MPFR_RNDA, MPFR_RNDD, MPFR_RNDN for negative y;
+   for the last test MPFR_RNDN, the target precision is decreased by 1 in
+   order to be able to deduce the result (anyway, for a hard-to-round case
+   in directed rounding modes, if yprec is chosen to be minimum precision
+   preserving this hard-to-round case, then one has a hard-to-round case
+   in round-to-nearest for precision yprec-1). If the target precision was
+   MPFR_PREC_MIN, then skip the MPFR_RNDN test; thus to test exact special
+   cases, use a target precision larger than MPFR_PREC_MIN.
+   Note: if y is a regular number, sb corresponds to the sticky bit when
+   considering round-to-nearest with precision yprec-1.
    As examples of use, see the calls to test5rm from the data_check and
    bad_cases functions. */
 static void
 test5rm (int (*fct) (FLIST), mpfr_srcptr x, mpfr_ptr y, mpfr_ptr z,
-         mpfr_rnd_t rnd, int test_one, const char *name)
+         mpfr_rnd_t rnd, int sb, const char *name)
 {
   mpfr_prec_t yprec = MPFR_PREC (y);
   mpfr_rnd_t rndnext = MPFR_RND_MAX;  /* means uninitialized */
+  int expected_inex = INT_MIN;
 
-  MPFR_ASSERTN (test_one || rnd == MPFR_RNDZ);
+  MPFR_ASSERTN (sb == -1 || rnd == MPFR_RNDZ);
   mpfr_set_prec (z, yprec);
   while (1)
     {
@@ -691,7 +740,13 @@ test5rm (int (*fct) (FLIST), mpfr_srcptr x, mpfr_ptr y, mpfr_ptr z,
 
       MPFR_ASSERTN (rnd != MPFR_RND_MAX);
       inex = fct (z, x, rnd);
-      if (! SAME_VAL (y, z))
+      if (sb == -1)
+        expected_inex = inex;  /* not checked */
+      else if (rnd != MPFR_RNDN)
+        expected_inex =
+          sb == 0 ? 0 : MPFR_IS_LIKE_RNDD (rnd, MPFR_SIGN (y)) ? -1 : 1;
+      MPFR_ASSERTN (expected_inex != INT_MIN);
+      if (!(SAME_VAL (y, z) || SAME_SIGN (inex, expected_inex)))
         {
           printf ("Error for %s with xprec=%lu, yprec=%lu, rnd=%s\nx = ",
                   name, (unsigned long) MPFR_PREC (x), (unsigned long) yprec,
@@ -702,30 +757,13 @@ test5rm (int (*fct) (FLIST), mpfr_srcptr x, mpfr_ptr y, mpfr_ptr z,
           printf ("\ngot      ");
           mpfr_out_str (stdout, 16, 0, z, MPFR_RNDN);
           printf ("\n");
+          if (sb != -1)
+            printf ("Expected inex = %d, got %d\n", expected_inex, inex);
           exit (1);
         }
-      if (test_one == 2 && inex != 0)
-        {
-          printf ("Error for %s with xprec=%lu, yprec=%lu, rnd=%s\nx = ",
-                  name, (unsigned long) MPFR_PREC (x), (unsigned long) yprec,
-                  mpfr_print_rnd_mode (rnd));
-          mpfr_out_str (stdout, 16, 0, x, MPFR_RNDN);
-          printf ("\nexact case, but non-zero ternary value (%d)\n", inex);
-          exit (1);
-        }
-      if (rnd == MPFR_RNDN)
+
+      if (sb == -1 || rnd == MPFR_RNDN)
         break;
-
-      if (test_one)
-        {
-          if (rnd == MPFR_RNDU || rnd == MPFR_RNDD)
-            break;
-
-          if (MPFR_IS_NEG (y))
-            rnd = (rnd == MPFR_RNDA) ? MPFR_RNDD : MPFR_RNDU;
-          else
-            rnd = (rnd == MPFR_RNDA) ? MPFR_RNDU : MPFR_RNDD;
-        }
       else if (rnd == MPFR_RNDZ)
         {
           rnd = MPFR_IS_NEG (y) ? MPFR_RNDU : MPFR_RNDD;
@@ -736,8 +774,9 @@ test5rm (int (*fct) (FLIST), mpfr_srcptr x, mpfr_ptr y, mpfr_ptr z,
           rnd = rndnext;
           if (rnd == MPFR_RNDA)
             {
-              mpfr_nexttoinf (y);
-              rndnext = (MPFR_IS_NEG (y)) ? MPFR_RNDD : MPFR_RNDU;
+              if (sb)
+                mpfr_nexttoinf (y);
+              rndnext = MPFR_IS_NEG (y) ? MPFR_RNDD : MPFR_RNDU;
             }
           else if (rndnext != MPFR_RNDN)
             rndnext = MPFR_RNDN;
@@ -745,7 +784,14 @@ test5rm (int (*fct) (FLIST), mpfr_srcptr x, mpfr_ptr y, mpfr_ptr z,
             {
               if (yprec == MPFR_PREC_MIN)
                 break;
-              mpfr_prec_round (y, --yprec, MPFR_RNDZ);
+              /* If sb = 1, then mpfr_nexttoinf was called on y for the
+                 MPFR_RNDA test, i.e. y = RNDA(yprec,f(x)); we use MPFR_RNDZ
+                 since one has the property RNDN(p,w) = RNDZ(p,RNDA(p+1,w))
+                 when w is not a midpoint in precision p. If sb = 0, then
+                 y = f(x), so that RNDN(yprec-1,f(x)) = RNDN(yprec-1,y). */
+              inex = mpfr_prec_round (y, --yprec, sb ? MPFR_RNDZ : MPFR_RNDN);
+              expected_inex = sb ?
+                MPFR_SIGN (y) * (inex == 0 ? 1 : -1) : inex;
               mpfr_set_prec (z, yprec);
             }
         }
@@ -765,14 +811,14 @@ test5rm (int (*fct) (FLIST), mpfr_srcptr x, mpfr_ptr y, mpfr_ptr z,
    x is the input (hexadecimal format)
    y is the expected output (hexadecimal format) for foo(x) with rounding rnd
 
-   If rnd is Z, y is the expected output in round-toward-zero, and the
-   four directed rounding modes are tested, then the round-to-nearest
-   mode is tested in precision yprec-1. This is useful for worst cases,
-   where yprec is the minimum value such that one has a worst case in a
-   directed rounding mode.
+   If rnd is Z, then y is the expected output in round-toward-zero and
+   it is assumed to be inexact; the four directed rounding modes are
+   tested, and the round-to-nearest mode is tested in precision yprec-1.
+   See details in the description of test5rm above.
 
-   If rnd is *, y must be an exact case. All the rounding modes are tested
-   and the ternary value is checked (it must be 0).
+   If rnd is *, y must be an exact case (possibly a special case).
+   All the rounding modes are tested and the ternary value is checked
+   (it must be 0).
  */
 void
 data_check (const char *f, int (*foo) (FLIST), const char *name)
@@ -906,13 +952,9 @@ data_check (const char *f, int (*foo) (FLIST), const char *name)
               exit (1);
             }
           if (r == '*')
-            {
-              int rndint;
-              RND_LOOP (rndint)
-                test5rm (foo, x, y, z, (mpfr_rnd_t) rndint, 2, name);
-            }
+            test5rm (foo, x, y, z, MPFR_RNDZ, 0, name);
           else
-            test5rm (foo, x, y, z, rnd, r != 'Z', name);
+            test5rm (foo, x, y, z, rnd, r == 'Z' ? 1 : -1, name);
         }
     }
 
@@ -945,7 +987,7 @@ bad_cases (int (*fct)(FLIST), int (*inv)(FLIST), const char *name,
 {
   mpfr_t x, y, z;
   char *dbgenv;
-  int i, dbg;
+  int cnt = 0, i, dbg;
   mpfr_exp_t old_emin, old_emax;
 
   old_emin = mpfr_get_emin ();
@@ -953,14 +995,14 @@ bad_cases (int (*fct)(FLIST), int (*inv)(FLIST), const char *name,
 
   dbgenv = getenv ("MPFR_DEBUG_BADCASES");
   dbg = dbgenv != 0 ? atoi (dbgenv) : 0;  /* debug level */
-  mpfr_inits (x, y, z, (mpfr_ptr) 0);
+  mpfr_inits2 (MPFR_PREC_MIN, x, y, z, (mpfr_ptr) 0);
   for (i = 0; i < n; i++)
     {
       mpfr_prec_t px, py, pz;
-      int inex;
+      int inex_inv, inex, sb;
 
       if (dbg)
-        printf ("bad_cases: i = %d\n", i);
+        printf ("bad_cases: %s, i = %d\n", name, i);
       py = pymin + (randlimb () % (pymax - pymin + 1));
       mpfr_set_prec (y, py);
       tests_default_random (y, pos, emin, emax, 0);
@@ -972,8 +1014,10 @@ bad_cases (int (*fct)(FLIST), int (*inv)(FLIST), const char *name,
         }
       px = py + psup;
       mpfr_set_prec (x, px);
+      if (dbg)
+        printf ("bad_cases: xprec =%4ld\n", (long) px);
       mpfr_clear_flags ();
-      inv (x, y, MPFR_RNDN);
+      inex_inv = inv (x, y, MPFR_RNDN);
       if (mpfr_nanflag_p () || mpfr_overflow_p () || mpfr_underflow_p ())
         {
           if (dbg)
@@ -991,11 +1035,20 @@ bad_cases (int (*fct)(FLIST), int (*inv)(FLIST), const char *name,
         {
           pz += 32;
           mpfr_set_prec (z, pz);
-          if (fct (z, x, MPFR_RNDN) == 0)
+          sb = fct (z, x, MPFR_RNDN) != 0;
+          if (!sb)
             {
               if (dbg)
                 printf ("bad_cases: exact case\n");
-              goto next_i;
+              if (inex_inv)
+                {
+                  printf ("bad_cases: f exact while f^(-1) inexact,\n"
+                          "due to a poor choice of the parameters.\n");
+                  exit (1);
+                  /* alternatively, goto next_i */
+                }
+              inex = 0;
+              break;
             }
           if (dbg)
             {
@@ -1016,7 +1069,14 @@ bad_cases (int (*fct)(FLIST), int (*inv)(FLIST), const char *name,
               || ! mpfr_equal_p (z, y))
             {
               if (dbg)
-                printf ("bad_cases: inverse doesn't match\n");
+                {
+                  printf ("bad_cases: inverse doesn't match for %s\ny = ",
+                          name);
+                  mpfr_out_str (stdout, 16, 0, y, MPFR_RNDN);
+                  printf ("\nz = ");
+                  mpfr_out_str (stdout, 16, 0, z, MPFR_RNDN);
+                  printf ("\n");
+                }
               goto next_i;
             }
         }
@@ -1047,7 +1107,8 @@ bad_cases (int (*fct)(FLIST), int (*inv)(FLIST), const char *name,
           printf ("\n");
         }
       /* Note: y is now the expected result rounded toward zero. */
-      test5rm (fct, x, y, z, MPFR_RNDZ, 0, name);
+      test5rm (fct, x, y, z, MPFR_RNDZ, sb, name);
+      cnt++;
     next_i:
       /* In case the exponent range has been changed by
          tests_default_random()... */
@@ -1055,6 +1116,16 @@ bad_cases (int (*fct)(FLIST), int (*inv)(FLIST), const char *name,
       mpfr_set_emax (old_emax);
     }
   mpfr_clears (x, y, z, (mpfr_ptr) 0);
+
+  if (dbg)
+    printf ("bad_cases: %d bad cases over %d generated values\n", cnt, n);
+
+  if (getenv ("MPFR_CHECK_BADCASES") && n - cnt > n/10)
+    {
+      printf ("bad_cases: too few bad cases (%d over %d generated values)\n",
+              cnt, n);
+      exit (1);
+    }
 }
 
 void
@@ -1108,3 +1179,16 @@ tests_expect_abort (void)
   exit (77);
 #endif
 }
+
+/* Guess whether the test runs within Valgrind. */
+int
+tests_run_within_valgrind (void)
+{
+  char *p;
+
+  p = getenv ("LD_PRELOAD");
+  if (p == NULL)
+    return 0;
+  return (strstr (p, "/valgrind/") != NULL ||
+          strstr (p, "/vgpreload") != NULL);
+}
