@@ -1,6 +1,6 @@
 /* Test file for gamma function
 
-Copyright 2001-2019 Free Software Foundation, Inc.
+Copyright 2001-2020 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -326,11 +326,11 @@ special_overflow (void)
   mpfr_set_prec (y, 38);
   mpfr_set_str (x, "-2993155353253689176481146537402947624254601559176535", 10,
                 MPFR_RNDN);
-  mpfr_div_2exp (x, x, 170, MPFR_RNDN);
+  mpfr_div_2ui (x, x, 170, MPFR_RNDN);
   mpfr_gamma (y, x, MPFR_RNDN);
   mpfr_set_prec (x, 38);
   mpfr_set_str (x, "201948391737", 10, MPFR_RNDN);
-  mpfr_mul_2exp (x, x, 92, MPFR_RNDN);
+  mpfr_mul_2ui (x, x, 92, MPFR_RNDN);
   if (mpfr_cmp (x, y))
     {
       printf ("Error for gamma (test 5)\n");
@@ -1055,6 +1055,48 @@ exp_lgamma_tests (void)
   set_emax (emax);
 }
 
+/* Bug reported by Frithjof Blomquist on May 19, 2020.
+   For the record, this bug was present since r8981
+   (in mpfr_bernoulli_internal, den was wrongly reset to 1 in case
+   of failure in Ziv's loop). The bug only occurred up from r8986
+   where the initial precision was reduced, but was potentially
+   present in any case of failure of Ziv's loop. */
+static void
+bug20200519 (void)
+{
+  mpfr_prec_t prec = 25093;
+  mpfr_t x, y, z, d;
+  double dd;
+  size_t min_memory_limit, old_memory_limit;
+
+  old_memory_limit = tests_memory_limit;
+  min_memory_limit = 24000000;
+  if (tests_memory_limit > 0 && tests_memory_limit < min_memory_limit)
+    tests_memory_limit = min_memory_limit;
+
+  mpfr_init2 (x, prec);
+  mpfr_init2 (y, prec);
+  mpfr_init2 (z, prec + 100);
+  mpfr_init2 (d, 24);
+  mpfr_set_d (x, 2.5, MPFR_RNDN);
+  mpfr_gamma (y, x, MPFR_RNDN);
+  mpfr_gamma (z, x, MPFR_RNDN);
+  mpfr_sub (d, y, z, MPFR_RNDN);
+  mpfr_mul_2si (d, d, prec - mpfr_get_exp (y), MPFR_RNDN);
+  dd = mpfr_get_d (d, MPFR_RNDN);
+  if (dd < -0.5 || 0.5 < dd)
+    {
+      printf ("Error in bug20200519: dd=%f\n", dd);
+      exit (1);
+    }
+  mpfr_clear (x);
+  mpfr_clear (y);
+  mpfr_clear (z);
+  mpfr_clear (d);
+
+  tests_memory_limit = old_memory_limit;
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -1086,6 +1128,11 @@ main (int argc, char *argv[])
 
   data_check ("data/gamma", mpfr_gamma, "mpfr_gamma");
 
+  /* this test takes about one minute */
+  if (getenv ("MPFR_CHECK_EXPENSIVE") != NULL &&
+      getenv ("MPFR_CHECK_LARGEMEM") != NULL)
+    bug20200519 ();
+
   tests_end_mpfr ();
   return 0;
 }
