@@ -1,7 +1,7 @@
 /* Test file for mpfr_get_flt and mpfr_set_flt
 
-Copyright 2009, 2010, 2011, 2012, 2013 Free Software Foundation, Inc.
-Contributed by the AriC and Caramel projects, INRIA.
+Copyright 2009-2020 Free Software Foundation, Inc.
+Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -17,29 +17,57 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdlib.h>
 #include <float.h>     /* for FLT_MIN */
+
 #include "mpfr-test.h"
+#include "ieee_floats.h"
+
+/* return non-zero iff f == g, and if both are zero check the sign */
+static int
+equal_flt (float f, float g)
+{
+  if (f != 0.0)
+    return f == g;
+  else if (g != 0)
+    return 0;
+  else /* f = g = 0: check they have the same sign */
+    {
+      int sf, sg;
+      mpfr_t z;
+      mpfr_init2 (z, MPFR_PREC_MIN);
+      mpfr_set_flt (z, f, MPFR_RNDN);
+      sf = mpfr_signbit (z);
+      mpfr_set_flt (z, g, MPFR_RNDN);
+      sg = mpfr_signbit (z);
+      mpfr_clear (z);
+      return !sf == !sg;
+    }
+}
 
 int
 main (void)
 {
   mpfr_t x, y;
-  float f, g, infp;
+  float f, g;
   int i;
+#if !defined(MPFR_ERRDIVZERO)
+  float infp;
+#endif
 
-  infp = (float) DBL_POS_INF;
+  tests_start_mpfr ();
+
+#if !defined(MPFR_ERRDIVZERO)
+  infp = (float) MPFR_DBL_INFP;
   if (infp * 0.5 != infp)
     {
       fprintf (stderr, "Error, FLT_MAX + FLT_MAX does not yield INFP\n");
       fprintf (stderr, "(this is probably a compiler bug, please report)\n");
       exit (1);
     }
-
-  tests_start_mpfr ();
+#endif
 
   mpfr_init2 (x, 24);
   mpfr_init2 (y, 24);
@@ -47,9 +75,10 @@ main (void)
 #if !defined(MPFR_ERRDIVZERO)
   mpfr_set_nan (x);
   f = mpfr_get_flt (x, MPFR_RNDN);
-  if (f == f)
+  if (! DOUBLE_ISNAN (f))
     {
       printf ("Error for mpfr_get_flt(NaN)\n");
+      printf ("got f=%f\n", f);
       exit (1);
     }
   mpfr_set_flt (x, f, MPFR_RNDN);
@@ -85,21 +114,23 @@ main (void)
   mpfr_set_ui (x, 0, MPFR_RNDN);
   f = mpfr_get_flt (x, MPFR_RNDN);
   mpfr_set_flt (x, f, MPFR_RNDN);
-  if (mpfr_zero_p (x) == 0 || MPFR_SIGN (x) < 0)
+  if (mpfr_zero_p (x) == 0 || MPFR_IS_NEG (x))
     {
       printf ("Error for mpfr_set_flt(mpfr_get_flt(+0))\n");
       exit (1);
     }
 
+#ifdef HAVE_SIGNEDZ
   mpfr_set_ui (x, 0, MPFR_RNDN);
   mpfr_neg (x, x, MPFR_RNDN);
   f = mpfr_get_flt (x, MPFR_RNDN);
   mpfr_set_flt (x, f, MPFR_RNDN);
-  if (mpfr_zero_p (x) == 0 || MPFR_SIGN (x) > 0)
+  if (mpfr_zero_p (x) == 0 || MPFR_IS_POS (x))
     {
       printf ("Error for mpfr_set_flt(mpfr_get_flt(-0))\n");
       exit (1);
     }
+#endif  /* HAVE_SIGNEDZ */
 
   mpfr_set_ui (x, 17, MPFR_RNDN);
   f = mpfr_get_flt (x, MPFR_RNDN);
@@ -137,7 +168,7 @@ main (void)
           printf ("got      "); mpfr_dump (y);
           exit (1);
         }
-      mpfr_mul_2exp (x, x, 1, MPFR_RNDN);
+      mpfr_mul_2ui (x, x, 1, MPFR_RNDN);
     }
 
   mpfr_set_prec (x, 53);
@@ -155,7 +186,7 @@ main (void)
           printf ("got      "); mpfr_dump (y);
           exit (1);
         }
-      mpfr_mul_2exp (x, x, 1, MPFR_RNDN);
+      mpfr_mul_2ui (x, x, 1, MPFR_RNDN);
     }
 
   mpfr_set_prec (x, 53);
@@ -173,124 +204,152 @@ main (void)
           printf ("got      "); mpfr_dump (y);
           exit (1);
         }
-      mpfr_mul_2exp (x, x, 1, MPFR_RNDN);
+      mpfr_mul_2ui (x, x, 1, MPFR_RNDN);
     }
 
-  mpfr_set_si_2exp (x, 1, -150, MPFR_RNDN);
-  g = 0.0;
-  f = mpfr_get_flt (x, MPFR_RNDN);
-  if (f != g)
+#ifdef HAVE_SUBNORM_FLT
+  for (i = 0; i < 2; i++)
     {
-      printf ("Error for mpfr_get_flt(2^(-150),RNDN)\n");
-      printf ("expected %.8e, got %.8e\n", g, f);
-      exit (1);
-    }
-  f = mpfr_get_flt (x, MPFR_RNDZ);
-  if (f != g)
-    {
-      printf ("Error for mpfr_get_flt(2^(-150),RNDZ)\n");
-      printf ("expected %.8e, got %.8e\n", g, f);
-      exit (1);
-    }
-  f = mpfr_get_flt (x, MPFR_RNDD);
-  if (f != g)
-    {
-      printf ("Error for mpfr_get_flt(2^(-150),RNDD)\n");
-      printf ("expected %.8e, got %.8e\n", g, f);
-      exit (1);
-    }
-  g = FLT_MIN * FLT_EPSILON;
-  f = mpfr_get_flt (x, MPFR_RNDU);
-  if (f != g)
-    {
-      printf ("Error for mpfr_get_flt(2^(-150),RNDU)\n");
-      printf ("expected %.8e, got %.8e\n", g, f);
-      exit (1);
-    }
-  f = mpfr_get_flt (x, MPFR_RNDA);
-  if (f != g)
-    {
-      printf ("Error for mpfr_get_flt(2^(-150),RNDA)\n");
-      printf ("expected %.8e, got %.8e\n", g, f);
-      exit (1);
-    }
+      mpfr_set_si_2exp (x, 1, -150, MPFR_RNDN);
+      g = 0.0;
+      if (i == 1)
+        {
+          mpfr_neg (x, x, MPFR_RNDN);
+          g = -g;
+        }
+      f = mpfr_get_flt (x, MPFR_RNDN);
+      if (!equal_flt (f, g))
+        {
+          printf ("Error for mpfr_get_flt(2^(-150),RNDN)\n");
+          printf ("expected %.8e, got %.8e\n", g, f);
+          exit (1);
+        }
+      f = mpfr_get_flt (x, MPFR_RNDZ);
+      if (!equal_flt (f, g))
+        {
+          printf ("Error for mpfr_get_flt(2^(-150),RNDZ)\n");
+          printf ("expected %.8e, got %.8e\n", g, f);
+          exit (1);
+        }
+      f = mpfr_get_flt (x, (i == 0) ? MPFR_RNDD : MPFR_RNDU);
+      if (!equal_flt (f, g))
+        {
+          printf ("Error for mpfr_get_flt(2^(-150),%s)\n",
+                  (i == 0) ? "RNDD" : "RNDU");
+          printf ("expected %.8e, got %.8e\n", g, f);
+          exit (1);
+        }
+      g = FLT_MIN * FLT_EPSILON;
+      if (i == 1)
+        g = -g;
+      f = mpfr_get_flt (x, (i == 0) ? MPFR_RNDU : MPFR_RNDD);
+      if (!equal_flt (f, g))
+        {
+          printf ("Error for mpfr_get_flt(2^(-150),%s)\n",
+                  (i == 0) ? "RNDU" : "RNDD");
+          printf ("expected %.8e, got %.8e\n", g, f);
+          exit (1);
+        }
+      f = mpfr_get_flt (x, MPFR_RNDA);
+      if (!equal_flt (f, g))
+        {
+          printf ("Error for mpfr_get_flt(2^(-150),RNDA)\n");
+          printf ("expected %.8e, got %.8e\n", g, f);
+          exit (1);
+        }
 
-  mpfr_set_si_2exp (x, 1, -151, MPFR_RNDN);
-  g = 0.0;
-  f = mpfr_get_flt (x, MPFR_RNDN);
-  if (f != g)
-    {
-      printf ("Error for mpfr_get_flt(2^(-151),RNDN)\n");
-      printf ("expected %.8e, got %.8e\n", g, f);
-      exit (1);
-    }
-  f = mpfr_get_flt (x, MPFR_RNDZ);
-  if (f != g)
-    {
-      printf ("Error for mpfr_get_flt(2^(-151),RNDZ)\n");
-      printf ("expected %.8e, got %.8e\n", g, f);
-      exit (1);
-    }
-  f = mpfr_get_flt (x, MPFR_RNDD);
-  if (f != g)
-    {
-      printf ("Error for mpfr_get_flt(2^(-151),RNDD)\n");
-      printf ("expected %.8e, got %.8e\n", g, f);
-      exit (1);
-    }
-  g = FLT_MIN * FLT_EPSILON;
-  f = mpfr_get_flt (x, MPFR_RNDU);
-  if (f != g)
-    {
-      printf ("Error for mpfr_get_flt(2^(-151),RNDU)\n");
-      printf ("expected %.8e, got %.8e\n", g, f);
-      exit (1);
-    }
-  f = mpfr_get_flt (x, MPFR_RNDA);
-  if (f != g)
-    {
-      printf ("Error for mpfr_get_flt(2^(-151),RNDA)\n");
-      printf ("expected %.8e, got %.8e\n", g, f);
-      exit (1);
-    }
+      mpfr_set_si_2exp (x, 1, -151, MPFR_RNDN);
+      g = 0.0;
+      if (i == 1)
+        {
+          mpfr_neg (x, x, MPFR_RNDN);
+          g = -g;
+        }
+      f = mpfr_get_flt (x, MPFR_RNDN);
+      if (!equal_flt (f, g))
+        {
+          printf ("Error for mpfr_get_flt(2^(-151),RNDN)\n");
+          printf ("expected %.8e, got %.8e\n", g, f);
+          exit (1);
+        }
+      f = mpfr_get_flt (x, MPFR_RNDZ);
+      if (!equal_flt (f, g))
+        {
+          printf ("Error for mpfr_get_flt(2^(-151),RNDZ)\n");
+          printf ("expected %.8e, got %.8e\n", g, f);
+          exit (1);
+        }
+      f = mpfr_get_flt (x, (i == 0) ? MPFR_RNDD : MPFR_RNDU);
+      if (!equal_flt (f, g))
+        {
+          printf ("Error for mpfr_get_flt(2^(-151),%s)\n",
+                  (i == 0) ? "RNDD" : "RNDU");
+          printf ("expected %.8e, got %.8e\n", g, f);
+          exit (1);
+        }
+      g = FLT_MIN * FLT_EPSILON;
+      if (i == 1)
+        g = -g;
+      f = mpfr_get_flt (x, (i == 0) ? MPFR_RNDU : MPFR_RNDD);
+      if (!equal_flt (f, g))
+        {
+          printf ("Error for mpfr_get_flt(2^(-151),%s)\n",
+                  (i == 0) ? "RNDU" : "RNDD");
+          printf ("expected %.8e, got %.8e\n", g, f);
+          exit (1);
+        }
+      f = mpfr_get_flt (x, MPFR_RNDA);
+      if (!equal_flt (f, g))
+        {
+          printf ("Error for mpfr_get_flt(2^(-151),RNDA)\n");
+          printf ("expected %.8e, got %.8e\n", g, f);
+          exit (1);
+        }
 
-  mpfr_set_si_2exp (x, 1, -149, MPFR_RNDN);
-  g = FLT_MIN * FLT_EPSILON;
-  f = mpfr_get_flt (x, MPFR_RNDN);
-  if (f != g)
-    {
-      printf ("Error for mpfr_get_flt(2^(-149),RNDN)\n");
-      printf ("expected %.8e, got %.8e\n", g, f);
-      exit (1);
-    }
-  f = mpfr_get_flt (x, MPFR_RNDZ);
-  if (f != g)
-    {
-      printf ("Error for mpfr_get_flt(2^(-149),RNDZ)\n");
-      printf ("expected %.8e, got %.8e\n", g, f);
-      exit (1);
-    }
-  f = mpfr_get_flt (x, MPFR_RNDD);
-  if (f != g)
-    {
-      printf ("Error for mpfr_get_flt(2^(-149),RNDD)\n");
-      printf ("expected %.8e, got %.8e\n", g, f);
-      exit (1);
-    }
-  f = mpfr_get_flt (x, MPFR_RNDU);
-  if (f != g)
-    {
-      printf ("Error for mpfr_get_flt(2^(-149),RNDU)\n");
-      printf ("expected %.8e, got %.8e\n", g, f);
-      exit (1);
-    }
-  f = mpfr_get_flt (x, MPFR_RNDA);
-  if (f != g)
-    {
-      printf ("Error for mpfr_get_flt(2^(-149),RNDA)\n");
-      printf ("expected %.8e, got %.8e\n", g, f);
-      exit (1);
+      mpfr_set_si_2exp (x, 1, -149, MPFR_RNDN);
+      g = FLT_MIN * FLT_EPSILON;
+      if (i == 1)
+        {
+          mpfr_neg (x, x, MPFR_RNDN);
+          g = -g;
+        }
+      f = mpfr_get_flt (x, MPFR_RNDN);
+      if (!equal_flt (f, g))
+        {
+          printf ("Error for mpfr_get_flt(2^(-149),RNDN)\n");
+          printf ("expected %.8e, got %.8e\n", g, f);
+          exit (1);
+        }
+      f = mpfr_get_flt (x, MPFR_RNDZ);
+      if (!equal_flt (f, g))
+        {
+          printf ("Error for mpfr_get_flt(2^(-149),RNDZ)\n");
+          printf ("expected %.8e, got %.8e\n", g, f);
+          exit (1);
+        }
+      f = mpfr_get_flt (x, MPFR_RNDD);
+      if (!equal_flt (f, g))
+        {
+          printf ("Error for mpfr_get_flt(2^(-149),RNDD)\n");
+          printf ("expected %.8e, got %.8e\n", g, f);
+          exit (1);
+        }
+      f = mpfr_get_flt (x, MPFR_RNDU);
+      if (!equal_flt (f, g))
+        {
+          printf ("Error for mpfr_get_flt(2^(-149),RNDU)\n");
+          printf ("expected %.8e, got %.8e\n", g, f);
+          exit (1);
+        }
+      f = mpfr_get_flt (x, MPFR_RNDA);
+      if (!equal_flt (f, g))
+        {
+          printf ("Error for mpfr_get_flt(2^(-149),RNDA)\n");
+          printf ("expected %.8e, got %.8e\n", g, f);
+          exit (1);
+        }
     }
+#endif /* HAVE_SUBNORM_FLT */
 
   mpfr_set_si_2exp (x, 1, 128, MPFR_RNDN);
   g = FLT_MAX;
@@ -308,7 +367,27 @@ main (void)
       printf ("expected %.8e, got %.8e\n", g, f);
       exit (1);
     }
+
+  /* same for negative x */
+  mpfr_set_si_2exp (x, -1, 128, MPFR_RNDN);
+  g = -FLT_MAX;
+  f = mpfr_get_flt (x, MPFR_RNDZ);
+  if (f != g)
+    {
+      printf ("Error for mpfr_get_flt(-2^128,RNDZ)\n");
+      printf ("expected %.8e, got %.8e\n", g, f);
+      exit (1);
+    }
+  f = mpfr_get_flt (x, MPFR_RNDU);
+  if (f != g)
+    {
+      printf ("Error for mpfr_get_flt(-2^128,RNDD)\n");
+      printf ("expected %.8e, got %.8e\n", g, f);
+      exit (1);
+    }
+
 #if !defined(MPFR_ERRDIVZERO)
+  mpfr_set_si_2exp (x, 1, 128, MPFR_RNDN);
   f = mpfr_get_flt (x, MPFR_RNDN); /* 2^128 rounds to itself with extended
                                       exponent range, we should get +Inf */
   g = infp;
@@ -332,6 +411,32 @@ main (void)
       printf ("expected %.8e, got %.8e\n", g, f);
       exit (1);
     }
+
+  /* same for negative x */
+  mpfr_set_si_2exp (x, -1, 128, MPFR_RNDN);
+  f = mpfr_get_flt (x, MPFR_RNDN); /* -2^128 rounds to itself with extended
+                                      exponent range, we should get +Inf */
+  g = -infp;
+  if (f != g)
+    {
+      printf ("Error for mpfr_get_flt(-2^128,RNDN)\n");
+      printf ("expected %.8e, got %.8e\n", g, f);
+      exit (1);
+    }
+  f = mpfr_get_flt (x, MPFR_RNDD);
+  if (f != g)
+    {
+      printf ("Error for mpfr_get_flt(-2^128,RNDD)\n");
+      printf ("expected %.8e, got %.8e\n", g, f);
+      exit (1);
+    }
+  f = mpfr_get_flt (x, MPFR_RNDA);
+  if (f != g)
+    {
+      printf ("Error for mpfr_get_flt(-2^128,RNDA)\n");
+      printf ("expected %.8e, got %.8e\n", g, f);
+      exit (1);
+    }
 #endif
 
   /* corner case: take x with 25 bits just below 2^128 */
@@ -353,6 +458,7 @@ main (void)
       printf ("expected %.8e, got %.8e\n", g, f);
       exit (1);
     }
+#if !defined(MPFR_ERRDIVZERO)
   f = mpfr_get_flt (x, MPFR_RNDN); /* first round to 2^128 (even rule),
                                       thus we should get +Inf */
   g = infp;
@@ -376,6 +482,7 @@ main (void)
       printf ("expected %.8e, got %.8e\n", g, f);
       exit (1);
     }
+#endif
 
   mpfr_clear (x);
   mpfr_clear (y);
