@@ -1,6 +1,6 @@
 /* Test file for mpfr_log1p.
 
-Copyright 2001-2017 Free Software Foundation, Inc.
+Copyright 2001-2020 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -17,12 +17,9 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdio.h>
-#include <stdlib.h>
-
 #include "mpfr-test.h"
 
 #ifdef CHECK_EXTERNAL
@@ -139,10 +136,10 @@ main (int argc, char *argv[])
   special ();
   other ();
 
-  test_generic (2, 100, 50);
+  test_generic (MPFR_PREC_MIN, 100, 50);
 
   data_check ("data/log1p", mpfr_log1p, "mpfr_log1p");
-  bad_cases (mpfr_log1p, mpfr_expm1, "mpfr_log1p", 256, -64, 40,
+  bad_cases (mpfr_log1p, mpfr_expm1, "mpfr_log1p", 256, -64, 9,
              4, 128, 800, 40);
 
   tests_end_mpfr ();
