@@ -1,6 +1,6 @@
 /* Test file for mpfr_mul_ui.
 
-Copyright 1999-2018 Free Software Foundation, Inc.
+Copyright 1999-2020 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -17,7 +17,7 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #include "mpfr-test.h"
@@ -87,9 +87,12 @@ main (int argc, char *argv[])
   unsigned int xprec, yprec, i;
   mpfr_prec_t p;
   mpfr_exp_t emax;
+  int r;
 
   tests_start_mpfr ();
 
+  emax = mpfr_get_emax ();
+
   for (p=2; p<100; p++)
     for (i=1; i<50; i++)
       check_inexact (p);
@@ -145,7 +148,6 @@ main (int argc, char *argv[])
   mpfr_mul_ui (x, x, 0, MPFR_RNDU);
   MPFR_ASSERTN(mpfr_cmp_ui (x, 0) == 0 && MPFR_IS_POS(x));
 
-  emax = mpfr_get_emax ();
   set_emax (0);
   mpfr_set_str_binary (x, "0.1E0");
   (mpfr_mul_ui) (x, x, 2, MPFR_RNDN);
@@ -279,6 +281,32 @@ main (int argc, char *argv[])
       exit (1);
     }
 
+  set_emax (MPFR_EMAX_MAX);
+  mpfr_set_prec (x, 32);
+  mpfr_set_prec (y, 96);
+  mpfr_set_inf (x, 1);
+  mpfr_nextbelow (x);
+  RND_LOOP (r)
+    {
+      mpfr_flags_t ex_flags, flags;
+
+      ex_flags = MPFR_FLAGS_OVERFLOW | MPFR_FLAGS_INEXACT;
+      mpfr_clear_flags ();
+      mpfr_mul_ui (y, x, 123, (mpfr_rnd_t) r);
+      flags = __gmpfr_flags;
+      if (flags != ex_flags)
+        {
+          printf ("Error in overflow check for %s\n",
+                  mpfr_print_rnd_mode ((mpfr_rnd_t) r));
+          printf ("Expected flags:");
+          flags_out (ex_flags);
+          printf ("Got:           ");
+          flags_out (flags);
+          exit (1);
+        }
+    }
+  set_emax (emax);
+
   mpfr_clear(x);
   mpfr_clear(y);
 
