@@ -1,10 +1,19 @@
 #!/bin/sh
 
+set -e
+
+if [ $# -ne 1 ]; then
+  echo "Usage: $0 <PATCHES_file>" >&2
+  exit 1
+fi
+
+patches=`cat "$1"`
+
 cat <<EOF
 /* mpfr_get_patches -- Patches that have been applied
 
-Copyright 2007-2015 Free Software Foundation, Inc.
-Contributed by the AriC and Caramel projects, INRIA.
+Copyright 2007-2020 Free Software Foundation, Inc.
+Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -20,7 +29,7 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #include "mpfr-impl.h"
@@ -30,5 +39,6 @@ mpfr_get_patches (void)
 {
 EOF
 
-echo '  return "'`cat PATCHES`'";'
+# $patches is written as a list of words separated by a space character.
+echo '  return "'$patches'";'
 echo '}'
